<?php
	// prevent direct call-up of include file
	if (realpath(__FILE__) == realpath($_SERVER['SCRIPT_FILENAME'])) die("
		/**<br>\n
		 * ts2query.php - Extracts data from a running Teamspeak server via the TCP query port.<br>\n
		 *<br>\n
		 * Release date and version number: February 26th, 2004. Ver 1.3<br>\n
		 *<br>\n
		 * Copyright (C) 2003  Christian Mller<br>\n
		 * <br>\n
		 * This program is free software; you can redistribute it and/or<br>\n
		 * modify it under the terms of the GNU General Public License<br>\n
		 * as published by the Free Software Foundation; either version 2<br>\n
		 * of the License, or (at your option) any later version.<br>\n
		 * <br>\n
		 * This program is distributed in the hope that it will be useful,<br>\n
		 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>\n
		 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br>\n
		 * GNU General Public License for more details.<br>\n
		 * <br>\n
		 * You should have received a copy of the GNU General Public License<br>\n
		 * along with this program; if not, write to the Free Software<br>\n
		 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.<br>\n
		 *<br>\n
		 * You may contact the author, Christian Mller, via electronic mail at<br>\n
		 * ts-query@brainspace.info<br>\n
		 *<br>\n
		 * A copy of the GNU GPL is provided in the file LICENSE<br>\n
		 *<br>\n
		 * Please read the CHANGELOG if you update from a previous version.<br>\n
		 *<br>\n
		 *<br>\n
		 * Usage of this script:<br>\n
		 * - Edit config.inc.php first!<br>\n
		 * - Include this script via an include directive.<br>\n
		 *   It will initialize automatically as $ts2info<br>\n
		 * - use the variables $ts2info->serverStatus, $ts2info->playerList, $ts2info->channelList<br>\n
		 *   to retrieve the information. Data structure information is provided further down<br>\n
		 *   in the variable documentation.<br>\n
		 */<br>\n
	");
?>
<?php
/**
 * ts2query.php - Extracts data from a running Teamspeak server via the TCP query port.
 *
 * Release date and version number: February 26th, 2004. Ver 1.3
 *
 * Copyright (C) 2003  Christian Mller
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * You may contact the author, Christian Mller, via electronic mail at
 * ts-query@brainspace.info
 *
 * A copy of the GNU GPL is provided in the file LICENSE
 *
 * Please read the CHANGELOG if you update from a previous version.
 *
 *
 * Usage of this script:
 * - Edit config.inc.php first!
 * - Include this script via an include directive.
 *   It will initialize automatically as $ts2info
 * - use the variables $ts2info->serverStatus, $ts2info->playerList, $ts2info->channelList
 *   to retrieve the information. Data structure information is provided further down
 *   in the variable documentation.
 */
class ts2query {
	// **** internal variables ****
	
	/**
	 * Server Address
	 *
	 * This information is used to connect to the Teamspeak server.
	 * If you want to connect to a Teamspeak server running on the same
	 * physical machine you should use 127.0.0.1, otherwise the IP
	 * address of the machine.
	 * If you attempt to connect to an external server you might need
	 * to increase the internal variable "timeout".
	 */
	var $serverAddress;
	
	/**
	 * Server Query Port
	 *
	 * This is the TCP query port of your Teamspeak server.
	 * Make sure it is correct and accessible, especially
	 * if you plan to use this script to access a remote
	 * Teamspeak server.
	 */
	var $serverQueryPort;
	
	/**
	 * Server UDP Port
	 *
	 * This is the port Teamspeak clients connect to.
	 * Although the query script doesn't use it it is a means
	 * of identifying a particular server if there is more than
	 * one Teamspeak server running on the same machine.
	 */
	var $serverUDPPort;

	/**
	 * Old PHP Version
	 *
	 * I'm developing on the current stable PHP version (4.3.4)
	 * If you experience problems try setting this variable
	 * to true. It might cost you some performance, but you WILL
	 * gain compatibility.
	 */
	var $OLD_PHP;


	// **** external variables ****
	
	/**
	 * Server Status
	 *
	 * This variable contains the server status.
	 * Its state can be "online" or "offline".
	 */
	var $serverStatus = "offline";

	/**
	 * Server Info
	 *
	 * This array contains the server information obtained from the query port.
	 * Field identifiers in the array correspond to the attribute names.
	 * The only field in use in my demo script is server_name.
	 */
	var $serverInfo = array();

	/**
	 * Global Info
	 *
	 * This array contains the global information obtained from the query port.
	 * Field identifiers in the array correspond to the attribute names.
	 */
	var $globalInfo = array();
	
	/**
	 * Player Data Structure
	 *
	 * This array contains the list of player data columns as obtained from the
	 * TCP query port.
	 */
	var $playerData = array();
	
	/**
	 * Player List
	 *
	 * This array contains the list of people currently on the server.
	 * Data structure is as follows for every entry in player list:
	 * 		playerID				player ID number
 	 * 		channelID				channel ID number
	 * 		receivedPackets			number of received packets (total for this session)
	 * 		receivedBytes			number of received bytes (total for this session)
	 * 		sentPackets				number of sent packets (total for this session)
	 * 		sentBytes				number of sent bytes (total for this session)
	 *  	packetLoss				packet loss so far
	 *  	ping					ping time of the player
	 * 		totalTime				total time on server
	 * 		idleTime				time since the player last said something
	 *		cprivs					channel privileges of the player, numerical form
	 *		channelPrivileges		channel privileges of the player, text
	 *			isChannelAdmin		is player channel admin?
	 *			hasAutoOperator		will player be made operator automatically?
	 *			hasAutoVoice		will player get voice automatically?
	 *			hasOperator			is the player an operator in this channel?
	 *			hasVoice			does the player have voice in this channel?
	 *		pprivs					global player privileges, numerical form
	 *		playerPrivileges		global player privileges, text form
	 *			isRegistered		is the player registered with the server?
	 *			isServerAdmin		is the player server admin?
	 *		pflags					player flags
	 *			isChannelCommander	is the player channel commander?
	 *			blocksWhispers		does the player block whispers?
	 *			isAway				is the player away?
	 *			mutedMicrophone		did the player mute his microphone?
	 *			mutedHeadphones		did the player mute his headphones?
	 *			isRecording			is the player recording?
	 *		playerIP				player's IP address
	 * 		playerName				nick name of the player
	 *		playerLoginName			the player's login name, if he is registered
	 */
	var $playerList = array();
	
	/**
	 * Channel List
	 *
	 * This array contains the list of channels for the server.
	 * Data structure is as follows for every entry in channel list:
	 *		channelID			channel ID number
	 *		codec				codec in use, numerical form
	 *		codecVerbose		codec in use, text form
	 *		parent				ID number of the parent channel, -1 if channel has no parent
	 *		order				ordering information
	 *		maxPlayers			maximum number of players allowed in this channel
	 *		channelname			name of the channel
	 *		flags				channel flags, numerical form
	 *			isUnregistered	is this an unregistered channel?
	 *			isModerated		is the channel moderated?
	 *			isPassworded	does the channel have a password?
	 *			hasSubchannels	does it have subchannels?
	 *			isDefault		is it the default channel?
	 *		password			password of the channel (not available here)
	 *		topic				topic of the channel
	 *		currentPlayers		number of players currently in the channel
	 */
	var $channelList = array();


	var $socket = NULL;
	var $timeout;


	// reads the configuration file and extracts the necessary information
	function readConfig() {
		include("config.inc.php");
		$this->serverAddress = $TS2_SERVER_ADDRESS;
		$this->serverQueryPort = $TS2_SERVER_QUERY_PORT;
		$this->serverUDPPort = $TS2_SERVER_UDP_PORT;
		$this->OLD_PHP = $OLD_PHP;
		$this->timeout = $TS2_TIMEOUT;
	}// end function readConfig
	
	// opens a connection to the teamspeak server
	function getSocket($host, $port, $timeout) {
		@$socket = fsockopen($host, $port, $errno, $errstr, $timeout);
		if($socket and fread($socket, 4) == "[TS]") {
			fgets($socket, 128);
			return $socket;
		}// end if
		return false;
	}// end function getSocket(...)

	// sends a query to the teamspeak server
	function sendQuery($query) {
		fputs($this->socket, $query."\n");
	}// end function sendQuery(...)

	// answer OK?
	function getOK() {
		$result = fread($this->socket, 2);
		fgets($this->socket, 128);
		return($result == "OK");
	}// end function getOK(...)

	// returns the codec name
	function getVerboseCodec($codec) {
		if($codec == 0) {
			$codec = "CELP 5.1 Kbit";
		} elseif($codec == 1) {
			$codec = "CELP 6.3 Kbit";
		} elseif($codec == 2) {
			$codec = "GSM 14.8 Kbit";
		} elseif($codec == 3) {
			$codec = "GSM 16.4 Kbit";
		} elseif($codec == 4) {
			$codec = "CELP Windows 5.2 Kbit";
		} elseif($codec == 5) {
			$codec = "Speex 3.4 Kbit";
		} elseif($codec == 6) {
			$codec = "Speex 5.2 Kbit";
		} elseif($codec == 7) {
			$codec = "Speex 7.2 Kbit";
		} elseif($codec == 8) {
			$codec = "Speex 9.3 Kbit";
		} elseif($codec == 9) {
			$codec = "Speex 12.3 Kbit";
		} elseif($codec == 10) {
			$codec = "Speex 16.3 Kbit";
		} elseif($codec == 11) {
			$codec = "Speex 19.5 Kbit";
		} elseif($codec == 12) {
			$codec = "Speex 25.9 Kbit";
		} else {
			$codec = "unknown (".$codec.")";
		}// end if
		return $codec;
	}// end function getVerboseCodec($codec);

	// user comparator function for sorting
	// sorts channels by name
	function compareChannelNames($channelA, $channelB) {
		// extract the names
		$nameA = $channelA[name];
		$nameB = $channelB[name];
		
		// remove all whitespace from the names
		$nameA = ereg_replace(' ', '', $nameA);
		$nameB = ereg_replace(' ', '', $nameB);
		
		// return comparison result
		return strnatcmp($nameA, $nameB);
	}// end function compareChannelNames($channelA, $channelB)
	
	// user comparator function for sorting
	// sorts channels by order
	function compareChannelOrders($channelA, $channelB) {
		if($channelA[order] == $channelB[order]) {
			return 0;
		} else {
			return ($channelA[order] < $channelB[order]) ? -1 : 1;
		}// end if
	}// end function compareChannelOrders($channelA, $channelB)

	// user comparator function for sorting
	// sorts players by name
	function comparePlayerNames($playerA, $playerB) {
		// extract the names
		$nameA = $playerA[name];
		$nameB = $playerB[name];
		
		// remove all whitespace from the names
		$nameA = ereg_replace(' ', '', $nameA);
		$nameB = ereg_replace(' ', '', $nameB);
		
		// return comparison result
		return strnatcmp($nameA, $nameB);
	}// end function compareChannelNames($playerA, $playerB)

	// user comparator function for sorting
	// sorts users by privileges
	function comparePlayerPrivileges($userA, $userB) {
		// server admin == highest
		$traitA = $userA[isServerAdmin];
		$traitB = $userB[isServerAdmin];
		if($traitA != $traitB) {
			return ($traitA < $traitB) ? 1 : -1;
		}// end if
		
		// operators == second highest
		$traitA = $userA[hasOperator];
		$traitB = $userB[hasOperator];
		if($traitA != $traitB) {
			return ($traitA < $traitB) ? 1 : -1;
		}// end if
		
		// voice == third highest
		$traitA = $userA[hasVoice];
		$traitB = $userB[hasVoice];
		if($traitA != $traitB) {
			return ($traitA < $traitB) ? 1 : -1;
		}// end if

		// registered == second lowest
		$traitA = $userA[isRegistered];
		$traitB = $userB[isRegistered];
		if($traitA != $traitB) {
			return ($traitA < $traitB) ? 1 : -1;
		}// end if

		// unregistered users are all equal
		return 0;
	}// end function comparePlayerPrivileges($userA, $userB)

	// since usort&co have become unstable i need to implement
	// a stable version of quicksort so i don't mess up the
	// alphabetical order when sorting after the numerical oder values
	function quickSort($list, $sortMethod) {
		// handle trivial cases first
		if(sizeof($list) == 0) return;
		if(sizeof($list) == 1) return $list;
		if(sizeof($list) == 2) {
			// retrieve the two entries
			reset($list);
			$a = current($list);
			next($list);
			$b = current($list);
			
			// compare
			if($sortMethod == "BY_NAME") {
				$comparison = $this->compareChannelNames($a, $b);
			} else if($sortMethod == "BY_ORDER") {
				$comparison = $this->compareChannelOrders($a, $b);
			} else if($sortMethod == "BY_PLAYERNAME") {
				$comparison = $this->comparePlayerNames($a, $b);
			} else if($sortMethod == "BY_PRIVILEGES") {
				$comparison = $this->comparePlayerPrivileges($a, $b);
			} else {
				die("Whoops, wrong sorting method, ".$sortMethod);
			}// end if
			
			if($comparison <= 0) {
				return $list;
			} else {
				// build result array
				$result = array();
				array_push($result, $b);
				array_push($result, $a);
				return $result;
			}// end if
		}// end if
		
		
		// now the interesting stuff...
		
		// prepare the sets
		$smallerThanPivot = array();
		$equalPivot = array();
		$greaterThanPivot = array();
		
		// get the pivot element
		reset($list);
		$pivotElement = current($list);

		// build the sets
		foreach($list as $key => $value) {
			if($sortMethod == "BY_NAME") {
				$comparison = $this->compareChannelNames($value, $pivotElement);
			} else if($sortMethod == "BY_ORDER") {
				$comparison = $this->compareChannelOrders($value, $pivotElement);
			} else if($sortMethod == "BY_PLAYERNAME") {
				$comparison = $this->comparePlayerNames($value, $pivotElement);
			} else if($sortMethod == "BY_PRIVILEGES") {
				$comparison = $this->comparePlayerPrivileges($value, $pivotElement);
			} else {
				die("Whoops, wrong sorting method, ".$sortMethod);
			}// end if

			if($comparison < 0) {
				array_push($smallerThanPivot, $value);
			} else if($comparison == 0) {
				array_push($equalPivot, $value);
			} else {
				array_push($greaterThanPivot, $value);
			}// end if
		}// end foreach
		
		// sort the smaller and greater elements
		$smallerThanPivot = $this->quickSort($smallerThanPivot, $sortMethod);
		$greaterThanPivot = $this->quickSort($greaterThanPivot, $sortMethod);

		// assemble
		$result = array();
		$result = array_merge($smallerThanPivot, $equalPivot);
		$result = array_merge($result, $greaterThanPivot);
		
		return $result;
	}// end function quickSort($list)

	// returns $data separated into an array with $key=>$value
	function separateInfo($data) {
		// get the position of the first equal sign
		$firstEqual = strpos($data, "=");
		
		// if there's none, return nothing
		if(!$firstEqual) return;
		
		// extract the key
		$key = substr($data, 0, $firstEqual);
		
		// extract the value
		$value = substr($data, $firstEqual + 1);
	
		// special treatment for the value
		if($this->OLD_PHP) {
			$value = ereg_replace("\"", "", $value);
			$value = ereg_replace("\'", "", $value);
			$value = ereg_replace("\n", "", $value);
			$value = ereg_replace("\r", "", $value);
			$value = htmlspecialchars($value);
		} else {
			$value = htmlspecialchars(trim($value, "\x22\x27\x0D\n"));
		}// end if
		
		// pack results into the return array
		$result = Array();
		$result[0] = $key;
		$result[1] = $value;
	
		return $result;
	}// end function separateInfo($data)

	// reads a line of player info, for PHP backwards-compatibility
	function readPlayerInfo() {
		unset($this->playerInfo);
		do {
			$input = fread($this->socket, 1);
			if($input != "\n" && $input != "\r") $this->playerInfo .= $input;
		} while($input != "\n");
	}// end function readPlayerInfo()
	
	// retrieves the next bit of player info, for PHP backwards-compatibility
	function retrievePI() {
		$pos = strpos($this->playerInfo, "\t");
		if(is_integer($pos)) {
			$result = substr($this->playerInfo, 0, $pos);
		} else {
			$result = $this->playerInfo;
		}// end if
		
		// cut out used info
		if(is_integer($pos)) {
			$this->playerInfo = substr($this->playerInfo, $pos + 1);
		} else {
			$this->playerInfo = "";
		}// end if

		return $result;		
	}// end function retrievePI()

	// reads a line of channel info, for PHP backwards-compatibility
	function readChannelInfo() {
		unset($this->channelInfo);
		do {
			$input = fread($this->socket, 1);
			if($input != "\n" && $input != "\r") $this->channelInfo .= $input;
		} while($input != "\n");
	}// end function readChannelInfo()

	// retrieves the next bit of channel info, for PHP backwards-compatibility
	function retrieveCI() {
		$pos = strpos($this->channelInfo, "\t");
		if(is_integer($pos)) {
			$result = substr($this->channelInfo, 0, $pos);
		} else {
			$result = $this->channelInfo;
		}// end if
		
		// cut out used info
		if(is_integer($pos)) {
			$this->channelInfo = substr($this->channelInfo, $pos + 1);
		} else {
			$this->channelInfo = "";
		}// end if

		return $result;		
	}// end function retrieveCI()


	
	function getInfo() {
		// ---=== main program ===---
		
		// load configuration
		$this->readConfig();
	
		// establish connection to teamspeak server
		$this->socket = $this->getSocket($this->serverAddress, $this->serverQueryPort, $this->timeout);
		if($this->socket == false) {
			return;
		}// end if
		$this->serverStatus = "online";
	
		// select the one and only running server on port 8767
		$this->sendQuery("sel ".$this->serverUDPPort);
	
		// retrieve answer "OK"
		if(!$this->getOK()) {
			$this->serverStatus = "offline";
			return;
		}// end if

		// retrieve server information
		$this->sendQuery("si");

		if($this->OLD_PHP) {
			// vintage PHP
			
			// build answer
			$counter = 0;
			do {
				// extract a line and save it in $result
				unset($result);
				do {
					$input = fread($this->socket, 1);
					if($input != "\n" && $input != "\r") $result .= $input;
				} while($input != "\n");
				
				// put $result into the $answer array
				$answer[$counter] = $result;
				$counter++;
			} while(substr($result, 0, 2) != "OK");
		} else {
			// retrieve complete answer and split it up
			$answer = fread($this->socket, 65535);
			$answer = explode("\n", $answer);
		}// end if
		
		// read server information
		$this->serverInfo = array();
		foreach($answer as $key => $data) {
			// extract information
			$serverInfoRaw = $this->separateInfo($data);
			
			// if "OK" is reached break off
			if($serverInfoRaw[0] == "OK") break;
			
			$this->serverInfo[$serverInfoRaw[0]] = $serverInfoRaw[1];
		}// end foreach


		// retrieve global information
		$this->sendQuery("gi");
		
		if($this->OLD_PHP) {
			// vintage PHP
			
			// build answer
			$counter = 0;
			do {
				// extract a line and save it in $result
				unset($result);
				do {
					$input = fread($this->socket, 1);
					if($input != "\n" && $input != "\r") $result .= $input;
				} while($input != "\n");
				
				// put $result into the $answer array
				$answer[$counter] = $result;
				$counter++;
			} while(substr($result, 0, 2) != "OK");
		} else {
			// retrieve complete answer and split it up
			$answer = fread($this->socket, 65535);
			$answer = explode("\n", $answer);
		}// end if

		// read global information
		$this->globalInfo = array();
		foreach($answer as $key => $data) {
			// extract raw information
			$globalInfoRaw = $this->separateInfo($data);
			
			// if "OK" is reached break off
			if($globalInfoRaw[0] == "OK") break;
			
			$this->globalInfo[$globalInfoRaw[0]] = $globalInfoRaw[1];
		}// end foreach
		
		
		// retrieve player list
		$this->sendQuery("pl");

		if($this->OLD_PHP) {
			// vintage PHP
			
			// read first line of answer which is the column designations
			$this->readPlayerInfo();
			
			// retrieve column information
			$counter = 0;
			while($this->playerInfo != "") {
				$this->playerData[$counter] = $this->retrievePI();
				$counter++;
			}// end while
			
			$answer[0] = "";
			$counter = 1;
			do {
				// extract a line and save it in $result
				unset($result);
				do {
					$input = fread($this->socket, 1);
					if($input != "\n" && $input != "\r") $result .= $input;
				} while($input != "\n");
				
				// put $result into the $answer array
				$answer[$counter] = $result;
				$counter++;
			} while(substr($result, 0, 2) != "OK");
		} else {
			// retrieve complete answer and split it up
			$answer = fread($this->socket, 65535);
			$answer = explode("\n", $answer);
		
			// retrieve column information
			$this->playerData = explode("\t", $answer[0]);

			// clear whitespaces from column information
			foreach($this->playerData as $key => $value) {
				$this->playerData[$key] = trim($value);
			}// end foreach
		}// end if
		
		// read player data
		$this->playerList = array();
		foreach($answer as $key => $data) {
			// skip first entry (the column info, we already got that)
			if($key == 0) continue;

			// extract raw information
			if($this->OLD_PHP) {
				$counter = 0;
				$this->playerInfo = $data."\n";
				while($this->playerInfo != "") {
					$playerInfoRaw[$counter] = $this->retrievePI();
					$counter++;
				}// end while
			} else {
				$playerInfoRaw = explode("\t", $data);
			}// end if

			// give player info meaningful identifiers		
			$playerInfo = array();
			foreach($this->playerData as $key => $columnName) {
				// strip quotes and unnecessary whitespaces
				if($this->OLD_PHP) {
					$playerInfoRaw[$key] = ereg_replace("\"", "", $playerInfoRaw[$key]);
					$playerInfoRaw[$key] = ereg_replace("\'", "", $playerInfoRaw[$key]);
					$playerInfoRaw[$key] = ereg_replace("\n", "", $playerInfoRaw[$key]);
					$playerInfoRaw[$key] = ereg_replace("\r", "", $playerInfoRaw[$key]);
				} else {
					$playerInfoRaw[$key] = trim($playerInfoRaw[$key], "\x22\x27\x0D\n");
				}// end if

				// play it safe, trim whitespaces, convert html characters
				$playerInfo[$columnName] = htmlspecialchars(trim($playerInfoRaw[$key]));
			}// end foreach

			// break loop if we reach end of answer
			if($playerInfo[p_id] == "OK") break;

			// build player privileges string
			$isServerAdmin = ($playerInfo[pprivs] & 1) > 0;
			$isRegistered = ($playerInfo[pprivs] & 4) > 0;
			
			unset($playerPrivileges);
			if($isRegistered) $playerPrivileges .= "R "; else $playerPrivileges .= "U ";
			if($isServerAdmin) $playerPrivileges .= "SA ";

			// cut the trailing space
			if($playerPrivileges) $playerPrivileges = substr($playerPrivileges, 0, strlen($playerPrivileges) - 1);

			// build channel privileges string
			$isChannelAdmin = ($playerInfo[cprivs] & 1) > 0;

			$hasOperator = ($playerInfo[cprivs] & 2) > 0;
			$hasVoice = ($playerInfo[cprivs] & 4) > 0;
			$hasAutoOperator = ($playerInfo[cprivs] & 8) > 0;
			$hasAutoVoice = ($playerInfo[cprivs] & 16) > 0;

			unset($channelPrivileges);			
			if($isChannelAdmin) $channelPrivileges .= "CA ";
			if($hasAutoOperator) $channelPrivileges .= "AO ";
			if($hasAutoVoice) $channelPrivileges .= "AV ";
			if($hasOperator) $channelPrivileges .= "O ";
			if($hasVoice) $channelPrivileges .= "V ";

			// cut the trailing space
			if($channelPrivileges) $channelPrivileges = substr($channelPrivileges, 0, strlen($channelPrivileges) - 1);

			// evaluate player flags
			$isChannelCommander = ($playerInfo[pflags] & 1) > 0;
			$blocksWhispers = ($playerInfo[pflags] & 4) > 0;
			$isAway = ($playerInfo[pflags] & 8) > 0;
			$mutedMicrophone = ($playerInfo[pflags] & 16) > 0;
			$mutedHeadphones = ($playerInfo[pflags] & 32) > 0;
			$isRecording = ($playerInfo[pflags] & 64) > 0;

			// feed additional info
			$playerInfo[channelPrivileges] = $channelPrivileges;
			$playerInfo[isChannelAdmin] = $isChannelAdmin;
			$playerInfo[hasAutoOperator] = $hasAutoOperator;
			$playerInfo[hasAutoVoice] = $hasAutoVoice;
			$playerInfo[hasOperator] = $hasOperator;
			$playerInfo[hasVoice] = $hasVoice;
			$playerInfo[playerPrivileges] = $playerPrivileges;
			$playerInfo[isRegistered] = $isRegistered;
			$playerInfo[isServerAdmin] = $isServerAdmin;
			$playerInfo[isChannelCommander] = $isChannelCommander;
			$playerInfo[blocksWhispers] = $blocksWhispers;
			$playerInfo[isAway] = $isAway;
			$playerInfo[mutedMicrophone] = $mutedMicrophone;
			$playerInfo[mutedHeadphones] = $mutedHeadphones;
			$playerInfo[isRecording] = $isRecording;

			// plug player info into player list
			$this->playerList[$playerInfo[p_id]] = $playerInfo;
		}// end while

		// retrieve channel list
		$this->sendQuery("cl");

		if($this->OLD_PHP) {
			// vintage PHP
			
			// read first line of answer which is the column designations
			$this->readChannelInfo();
			
			// retrieve column information
			$counter = 0;
			while($this->channelInfo != "") {
				$this->channelData[$counter] = $this->retrieveCI();
				$counter++;
			}// end while
			
			$answer[0] = "";
			$counter = 1;
			do {
				// extract a line and save it in $result
				unset($result);
				do {
					$input = fread($this->socket, 1);
					if($input != "\n" && $input != "\r") $result .= $input;
				} while($input != "\n");
				
				// put $result into the $answer array
				$answer[$counter] = $result;
				$counter++;
			} while(substr($result, 0, 2) != "OK");
		} else {
			// retrieve complete answer and split it up
			$answer = fread($this->socket, 65535);
			$answer = explode("\n", $answer);

			// retrieve column information
			$this->channelData = explode("\t", $answer[0]);

			// clear whitespaces from column information
			foreach($this->channelData as $key => $value) {
				$this->channelData[$key] = trim($value);
			}// end foreach
		}// end if
			
		// read channel data
		$this->channelList = array();
		foreach($answer as $key => $data) {
			// skip first entry (the column info, we already got that)
			if($key == 0) continue;

			// extract raw information
			if($this->OLD_PHP) {
				$counter = 0;
				$this->channelInfo = $data."\n";
				while($this->channelInfo != "") {
					$channelInfoRaw[$counter] = $this->retrieveCI();
					$counter++;
				}// end while
			} else {
				$channelInfoRaw = explode("\t", $data);
			}// end if

			// give player info meaningful identifiers		
			$channelInfo = array();
			foreach($this->channelData as $key => $columnName) {
				// strip quotes
				if($this->OLD_PHP) {
					$channelInfoRaw[$key] = ereg_replace("\"", "", $channelInfoRaw[$key]);
					$channelInfoRaw[$key] = ereg_replace("\'", "", $channelInfoRaw[$key]);
					$channelInfoRaw[$key] = ereg_replace("\n", "", $channelInfoRaw[$key]);
					$channelInfoRaw[$key] = ereg_replace("\r", "", $channelInfoRaw[$key]);
				} else {
					$channelInfoRaw[$key] = trim($channelInfoRaw[$key], "\x22\x27\x0D\n");
				}// end if

				// play it safe, trim whitespaces, convert html characters
				$channelInfo[$columnName] = htmlspecialchars(trim($channelInfoRaw[$key]));
			}// end foreach

			// break loop if we reach end of answer
			if($channelInfo[id] == "OK") break;

			// evaluate flags
			$isUnregistered = ($channelInfo[flags] & 1) > 0;
			$isModerated = ($channelInfo[flags] & 2) > 0;
			$isPassworded = $channelInfo[password] != 0;
			$hasSubchannels = ($channelInfo[flags] & 8) > 0;
			$isDefault = ($channelInfo[flags] & 16) > 0;

			// determine number of players in channel
			$playerCount = 0;
			foreach($this->playerList as $playerInfo) {
				if($playerInfo[c_id] == $channelInfo[id]) $playerCount++;
			}// end foreach

			$channelInfo[codecVerbose] = $this->getVerboseCodec($channelInfo[codec]);
			$channelInfo[isUnregistered] = $isUnregistered;
			$channelInfo[isModerated] = $isModerated;
			$channelInfo[isPassworded] = $isPassworded;
			$channelInfo[hasSubchannels] = $hasSubchannels;
			$channelInfo[isDefault] = $isDefault;
			$channelInfo[currentPlayers] = $playerCount;

			// plug channel info into channel list
			$this->channelList[$channelInfo[id]] = $channelInfo;
		}// end while

		// close connection to teamspeak server
		$this->sendQuery("quit");
		fclose($this->socket);


		// sort channels by name
		$this->channelList = $this->quickSort($this->channelList, "BY_NAME");
		
		// now sort channels by order
		$this->channelList = $this->quickSort($this->channelList, "BY_ORDER");

		// sort players by player name
		$this->playerList = $this->quickSort($this->playerList, "BY_PLAYERNAME");
		
		// sort players by privileges
		$this->playerList = $this->quickSort($this->playerList, "BY_PRIVILEGES");
		
		
		// if a list is not set make an empty list
		if(!$this->playerList) $this->playerList = array();
		if(!$this->channelList) $this->channelList = array();
	}// end getInfo()
}// class tss2info

$ts2info = new ts2query;
$ts2info->getInfo();
?>
